import gtk
import x11


class KeyBinding:
    """
    Class for setting up global desktop keybindings.
    """

    def __init__(self):

        # mapping: (keycode, modifiers) -> action
        self.__action_table = {}

        # proxy widget for receiving events from the root window
        self.__proxy = gtk.Invisible()
        self.__proxy.connect("key-press-event", self.__event_handler)
        self.__proxy.show()




    def __event_handler(self, src, event):

        key = (event.hardware_keycode, event.state)
        action = self.__action_table.get(key)

        if (action): action()
        
        
        

    def bind_key(self, keycode, modifiers, handler):

        self.__action_table[(keycode, modifiers)] = handler
        try:
            x11.grab_ungrab_key(self.__proxy.window, keycode, modifiers, True)
        except:
            pass


    def unbind_key(self, keycode, modifiers):

        try:
            del self.__action_table[(keycode, modifiers)]
        except KeyError:
            pass

        try:
            x11.grab_ungrab_key(self.__proxy.window, keycode, modifiers, False)
        except:
            pass



from ConfigFloat import ConfigFloat
from utils.datatypes import *


class ConfigInteger(ConfigFloat):

    def __init__(self, name, getter, setter, caller):

        ConfigFloat.__init__(self, name, getter, setter, caller,
                             int_only = True)

        self._register_property("value", TYPE_INT, self._setp_value,
                                self._getp, 0.0, doc = "Value")



#
# Python's shelve module comes broken on more and more systems, so we have to
# use our own backend. This makes it all more portable also.
#


_MAGIC = "GDESKLETS"


class _Backend(dict):

    def __init__(self, filename, load = True):

        self.__dbfile = filename

        dict.__init__(self)

        if (load):
            try:
                self.__load()
            except:
                pass


    def __load(self):

        import cPickle
        data = open(self.__dbfile, "r").read()[len(_MAGIC):]
        self.clear()
        self.update(cPickle.loads(data))


    def __save(self):

        import cPickle
        data = cPickle.dumps(self.copy())
        fd = open(self.__dbfile, "w")
        fd.write(_MAGIC)
        fd.write(data)
        fd.close()


    def sync(self):

        self.__save()


    def set_file(self, path):

        self.__dbfile = path
        
        
#
# Factory function for the backend.
#
def Backend(filename, *args):

    try:
        header = open(filename).read(len(_MAGIC))
    except:
        header = _MAGIC
        
    if (header == _MAGIC):
        backend = _Backend(filename)
    else:
        # migrate legacy stuff to new format (hello seb128!)
        import shelve
        legacy = shelve.open(filename, flag = "c", writeback = False)
        backend = _Backend(filename, load = False)
        backend.update(legacy)

    return backend



from ConfigWidget import ConfigWidget
from utils.datatypes import *
from utils import vfs

import gtk
import os

# the modifier keys, i.e. the keys which don't represent full keybindings on
# their own
_MODIFIERS = ("Alt_L",
              "Alt_R",
              "Control_L",
              "Control_R",
              "Escape",
              "Hyper_L",
              "Hyper_R",
              "Super_L",
              "Super_R",
              "Shift_L",
              "Shift_R")

class ConfigKeyBinding(ConfigWidget):

    def __init__(self, name, getter, setter, caller):

        ConfigWidget.__init__(self, name, getter, setter, caller)

        self._register_property("value", TYPE_LIST, self._setp_value,
                                self._getp, "", doc = "keycode and modifiers")


    def get_widgets(self):

        def f(src):
            if (src.get_active()):
                gtk.gdk.keyboard_grab(src.window)
                src.set_label("")

        self.__label = gtk.Label("")
        self.__label.show()
        align = gtk.Alignment(0.0, 0.0, 0.0, 0.0)
        align.show()
        align.add(self.__label)

        self.__button = gtk.ToggleButton("F12")
        self.__button.set_use_underline(False)
        self.__button.show()
        self.__button.connect("clicked", f)
        self.__button.connect("key-press-event", self.__on_change)

        return (align, self.__button)


    def __key2text(self, keycode, modifiers):

        keymap = gtk.gdk.keymap_get_default()
        keyval, nil, nil, nil = keymap.get_entries_for_keycode(keycode)[0]

        out = ""
        if (gtk.gdk.SHIFT_MASK & modifiers):
            out += "<Shift> "
        if (gtk.gdk.CONTROL_MASK & modifiers):
            out += "<Ctrl> "
        if (gtk.gdk.MOD1_MASK & modifiers):
            out += "<Alt> "

        if (out): out += "+ "
        out += "<" + gtk.gdk.keyval_name(keyval) + ">"

        return out


    def __on_change(self, src, event):

        if (src.get_active()):
            name = gtk.gdk.keyval_name(event.keyval)

            if (not name in _MODIFIERS):
                src.set_label(self.__key2text(event.hardware_keycode,
                                              event.state))
                gtk.gdk.keyboard_ungrab()
                src.set_active(False)
            
        self._set_config((event.hardware_keycode, int(event.state)))



    def _set_label(self, value): self.__label.set_text(value)
    def _set_enabled(self, value): self.__ebox.set_sensitive(value)


    def _setp_value(self, key, value):

        keycode, modifiers = value
        self.__button.set_label(self.__key2text(keycode, modifiers))
        
        self._set_config(value)
        self._setp(key, value)
